@@ -1,9 +1,24 @@
+# The contents of this file are subject to the Python Software Foundation
+# License Version 2.3 (the License).  You may not copy or use this file, in
+# either source code or executable form, except in compliance with the License.
+# You may obtain a copy of the License at http://www.python.org/license.
+#
+# Software distributed under the License is distributed on an AS IS basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
+# for the specific language governing rights and limitations under the
+# License.
+
 import os
 import sys
 try:
     from resource import setrlimit, getrlimit, RLIMIT_NOFILE
+    import resource
+
     try:
+      try:
         setrlimit(RLIMIT_NOFILE, (100000, 100000))
+      except resource.error, e:
+        print ">>> unable to setrlimit ", e
     except ValueError, e:
         print ">>> unable to setrlimit ", e
 except ImportError, e:
