@@ -32,12 +32,20 @@ import urlparse
 
 ONLY_LOCAL = False
 
+GLOBAL_FILTER = None
+def GLOBAL_FILTER(ip, port, direction=""):
+    #print ip, direction
+    return False
+GLOBAL_FILTER = None
+
 # header, reserved, download id, my id, [length, message]
 
 LOWER_BOUND = 1
 UPPER_BOUND = 120
 BUFFER = 1.2
+use_timeout_order = False
 timeout_order = [3, 15, 30]
+debug = False
 
 def set_timeout_metrics(delta):
     delta = max(delta, 0.0001)
@@ -52,7 +60,7 @@ def set_timeout_metrics(delta):
 
 class GaurdedInitialConnection(Handler):
     def __init__(self, parent, id, encrypt=False, log_prefix="", lan=False,
-                 urgent=False, timeout=None):
+                 urgent=False, timeout=None ):
         self.t = None
         self.id = id
         self.lan = lan
@@ -86,10 +94,14 @@ class GaurdedInitialConnection(Handler):
         self.t = None        
         
     def connection_made(self, s):
-        set_timeout_metrics(bttime() - self.start)
-
+        t = bttime() - self.start
+        set_timeout_metrics(t)
         addr = (s.ip, s.port)
-        #print 'connection made', addr
+
+        if debug:        
+            self.parent.logger.warning('connection made: %s %s' %
+                                       (addr, t))
+
         del self.parent.pending_connections[addr]
 
         self._abort_timeout()        
@@ -102,8 +114,10 @@ class GaurdedInitialConnection(Handler):
         self.parent.replace_connection()
         
     def connection_failed(self, s, exception):
-        #print 'connection failed', addr, exception.getErrorMessage()
         addr = (s.ip, s.port)
+        if debug:
+            self.parent.logger.warning('connection failed: %s %s' %
+                                       (addr, exception.getErrorMessage()))
 
         if s.connector.wasPreempted():
             self.parent._resubmit_connection(addr)
@@ -139,8 +153,9 @@ class ConnectionManager(InternetSubscriber):
     def __init__(self, make_upload, downloader, choker,
                  numpieces, ratelimiter,
                  rawserver, config, private, my_id, add_task, infohash, context,
-                 addcontactfunc, reported_port, tracker_ips, log_prefix):
+                 addcontactfunc, reported_port, tracker_ips, log_prefix ): 
         """
+            @param downloader: MultiDownload for this torrent.
             @param my_id: my peer id.
             @param tracker_ips: list of tracker ip addresses.
                ConnectionManager does not drop connections from the tracker.
@@ -202,6 +217,8 @@ class ConnectionManager(InternetSubscriber):
         self.reopen(reported_port)        
 
     def cleanup(self):
+        if not self.closed:
+            self.close_connections()
         del self.context
         self.cached_peers.clear()
         if self._ka_task.active():
@@ -352,6 +369,9 @@ class ConnectionManager(InternetSubscriber):
         if ONLY_LOCAL and addr[0] != "127.0.0.1" and not addr[0].startswith("192.168") and addr[1] != 80:
             return True
 
+        if GLOBAL_FILTER and not GLOBAL_FILTER(addr[0], addr[1], "out"):
+            return True
+
         if addr not in self.cached_peers:
             self.cache_incomplete_peer(addr, pid, handler, *a, **kw)
 
@@ -362,7 +382,10 @@ class ConnectionManager(InternetSubscriber):
             return True
 
         kw['log_prefix'] = self.log_prefix
-        kw.setdefault('timeout', timeout_order[0])
+        timeout = 30
+        if use_timeout_order:
+            timeout = timeout_order[0]
+        kw.setdefault('timeout', timeout)
         h = handler(self, pid, *a, **kw)
         self.pending_connections[addr] = (h, (addr, pid, handler, a, kw))
         urgent = kw.pop('urgent', False)
@@ -393,7 +416,7 @@ class ConnectionManager(InternetSubscriber):
         h, info = self.pending_connections[addr]
         addr, pid, handler, a, kw = info
 
-        if h.timeout < timeout_order[-1]:
+        if use_timeout_order and h.timeout < timeout_order[-1]:
             for t in timeout_order:
                 if t > h.timeout:
                     h.timeout = t
@@ -506,6 +529,8 @@ class ConnectionManager(InternetSubscriber):
             connector.ip not in self.tracker_ips):
             return False
         self._add_connection(connector)
+        if self.closed:
+            return False
         connector.set_parent(self)
         connector.connection.context = self.context
         return True
@@ -558,7 +583,8 @@ class SingleportListener(Handler):
 
        See Connector which upcalls to select_torrent after the infohash is 
        received in the opening handshake."""
-    def __init__(self, rawserver, nattraverser, log_prefix):
+    def __init__(self, rawserver, nattraverser, log_prefix, 
+                 use_local_discovery):
         self.rawserver = rawserver
         self.nattraverser = nattraverser
         self.port = 0
@@ -570,7 +596,8 @@ class SingleportListener(Handler):
         self.obfuscated_torrents = {}
         self.local_discovery = None
         self.ld_services = {}
-        self._creating_local_discorvery = False
+        self.use_local_discovery = use_local_discovery
+        self._creating_local_discovery = False
         self.log_prefix = log_prefix
         self.logger = logging.getLogger(self.log_prefix)
 
@@ -624,14 +651,16 @@ class SingleportListener(Handler):
 
         if self.local_discovery:
             self.local_discovery.stop()
-        self._create_local_discovery()
+        if self.use_local_discovery:
+            self._create_local_discovery()
 
     def _create_local_discovery(self):
-        self._creating_local_discorvery = True
+        assert self.use_local_discovery
+        self._creating_local_discovery = True
         try:
             self.local_discovery = LocalDiscovery(self.rawserver, self.port,
                                                   self._start_connection)
-            self._creating_local_discorvery = False
+            self._creating_local_discovery = False
         except:
             self.rawserver.add_task(5, self._create_local_discovery)
 
@@ -695,6 +724,10 @@ class SingleportListener(Handler):
     def connection_made(self, connection):
         """Called when TCP connection has finished opening, but before
            BitTorrent protocol has begun."""
+        if ONLY_LOCAL and connection.ip != '127.0.0.1' and not connection.ip.startswith("192.168") :
+            return
+        if GLOBAL_FILTER and not GLOBAL_FILTER(connection.ip, connection.port, "in"):
+            return
         connector = Connector(self, connection, None, False,
                               log_prefix=self.log_prefix)
         self.connectors.add(connector)
@@ -702,8 +735,6 @@ class SingleportListener(Handler):
     def select_torrent(self, connector, infohash):
         """Called when infohash has been received allowing us to map
            the connection on to a given Torrent's ConnectionManager."""
-        if ONLY_LOCAL and connector.connection.ip != '127.0.0.1' and not connector.connection.ip.startswith("192.168") :
-            return
         # call-up from Connector.
         if infohash in self.torrents:
             accepted = self.torrents[infohash].singleport_connection(connector)
@@ -723,6 +754,8 @@ class SingleportListener(Handler):
         self.obfuscated_torrents[streamid].singleport_connection(connector)
 
     def connection_lost(self, connector):
+        if (ONLY_LOCAL or GLOBAL_FILTER) and connector not in self.connectors:
+            return
         assert isinstance(connector, Connector)
         self.connectors.remove(connector)
 
