@@ -35,8 +35,8 @@ from BitTorrent.DownloadRateLimiter import DownloadRateLimiter
 from BitTorrent.ConnectionManager import SingleportListener
 from BitTorrent.CurrentRateMeasure import Measure
 from BitTorrent.Storage import FilePool
-from BTL.yielddefer import launch_coroutine, _wrap_task
-from BTL.defer import Deferred, DeferredEvent
+from BTL.yielddefer import launch_coroutine
+from BTL.defer import Deferred, DeferredEvent, wrap_task
 from BitTorrent import BTFailure, InfoHashType
 from BitTorrent import configfile
 from khashmir.utkhashmir import UTKhashmir
@@ -143,7 +143,8 @@ class MultiTorrent(Feedback):
         self.internet_watcher = get_internet_watcher(self.rawserver)
         self.singleport_listener = SingleportListener(self.rawserver,
                                                       nattraverser,
-                                                      self.log_root)
+                                                      self.log_root,
+                                                      config['use_local_discovery'])
         self.choker = Choker(self.config, self.rawserver.add_task)
         self.up_ratelimiter = RateLimiter(self.rawserver.add_task)
         self.up_ratelimiter.set_parameters(config['max_upload_rate'],
@@ -178,7 +179,6 @@ class MultiTorrent(Feedback):
             get_rates=self.get_total_rates )
 
         self.rawserver.add_task(0, self.butle)
-        #raise Exception("blah")
 
 
     def butle(self):
@@ -234,7 +234,7 @@ class MultiTorrent(Feedback):
                               exc_info=exc_info)
 
     def shutdown(self):
-        df = launch_coroutine(_wrap_task(self.rawserver.add_task), self._shutdown)
+        df = launch_coroutine(wrap_task(self.rawserver.add_task), self._shutdown)
         df.addErrback(lambda f : self.logger.error('shutdown failed!',
                                                    exc_info=f.exc_info()))
         return df
@@ -685,7 +685,7 @@ class MultiTorrent(Feedback):
         return df
 
     def initialize_torrents(self):
-        df = launch_coroutine(_wrap_task(self.rawserver.add_task), self._initialize_torrents)
+        df = launch_coroutine(wrap_task(self.rawserver.add_task), self._initialize_torrents)
         df.addErrback(lambda f : self.logger.error('initialize_torrents failed!',
                                                    exc_info=f.exc_info()))
         return df
