@@ -32,14 +32,14 @@ from BTL.platform import bttime, get_filesystem_encoding
 from BitTorrent.ConnectionManager import ConnectionManager
 from BitTorrent import PeerID
 from BTL.exceptions import str_exc
-from BTL.defer import ThreadedDeferred, Failure
-from BTL.yielddefer import launch_coroutine, _wrap_task
+from BTL.defer import ThreadedDeferred, Failure, wrap_task
+from BTL.yielddefer import launch_coroutine
 from BitTorrent.TorrentStats import TorrentStats
 from BitTorrent.RateMeasure import RateMeasure
 from BitTorrent.PiecePicker import PiecePicker
 from BitTorrent.Rerequester import Rerequester, DHTRerequester
 from BitTorrent.CurrentRateMeasure import Measure
-from BitTorrent.Storage import Storage
+from BitTorrent.Storage import Storage, UnregisteredFileException
 from BitTorrent.HTTPConnector import URLage
 from BitTorrent.StorageWrapper import StorageWrapper
 from BitTorrent.RequestManager import RequestManager
@@ -237,11 +237,18 @@ class Torrent(object):
 
     def _set_completed(self, val):
         self._completed = val
-        self.config['finishtime'] = bttime()
+        if val:
+            self.config['finishtime'] = bttime()
     def _get_completed(self):
         return self._completed
     completed = property(_get_completed, _set_completed)
 
+    def _set_sent_completed(self, value):
+        self.config['sent_completed'] = value
+    def _get_sent_completed(self):
+        return self.config['sent_completed']
+    sent_completed = property(_get_sent_completed, _set_sent_completed)
+
     def _get_finishtime(self):
         return self.config['finishtime']
     finishtime = property(_get_finishtime)
@@ -361,7 +368,7 @@ class Torrent(object):
         self.context_valid = True
         assert self.state in ["created", "failed", "finishing"], "state not in set"
         self.state = "initializing"
-        df = launch_coroutine(_wrap_task(self.add_task), self._initialize)
+        df = launch_coroutine(wrap_task(self.add_task), self._initialize)
         df.addErrback(self.got_exception)
         return df
 
@@ -503,7 +510,7 @@ class Torrent(object):
         md.add_useful_received_listener(self._total_downmeasure.update_rate)
         md.add_useful_received_listener(self._downmeasure.update_rate)
         md.add_useful_received_listener(self._ratemeasure.data_came_in)
-        md.add_useful_received_listener(self._down_ratelimiter.update_rate)
+        md.add_raw_received_listener(self._down_ratelimiter.update_rate)
         self.multidownload = md
 
         # HERE. Yipee! Uploads are created by callback while Download
@@ -522,7 +529,7 @@ class Torrent(object):
         else:
             addContact = None
 
-        df = self.metainfo.get_tracker_ips(_wrap_task(self.external_add_task))
+        df = self.metainfo.get_tracker_ips(wrap_task(self.external_add_task))
         yield df
         tracker_ips = df.getResult()
         self._connection_manager = \
@@ -569,9 +576,8 @@ class Torrent(object):
                                               self._connection_manager)
         self._listening = True
 
-        # the DHT is broken
-        #if self.metainfo.is_trackerless or not self.metainfo.is_private:
-        if self.metainfo.is_trackerless:
+        #if self.metainfo.is_trackerless:
+        if not self.metainfo.is_private:
             if not self._dht and self.metainfo.is_trackerless:
                 self._error(self, logging.CRITICAL,
                    _("Attempt to download a trackerless torrent "
@@ -681,7 +687,7 @@ class Torrent(object):
         # use _rawserver.add_task directly here, because we want the callbacks
         # to happen even though _shutdown is about to invalidate this torrent's
         # context
-        df = launch_coroutine(_wrap_task(self._rawserver.add_task), self._shutdown)
+        df = launch_coroutine(wrap_task(self._rawserver.add_task), self._shutdown)
         df.addErrback(self.got_exception, cannot_shutdown=True)
         return df
 
@@ -756,6 +762,10 @@ class Torrent(object):
         severity = logging.CRITICAL
         msg = "Torrent got exception: %s" % type
         e_str = str_exc(e)
+        if isinstance(e, UnregisteredFileException):
+            # not an error, a pending disk op was aborted because the torrent
+            # has unregistered files.
+            return
         if isinstance(e, BTFailure):
             self._activity = ( _("download failed: ") + e_str, 0)
         elif isinstance(e, IOError):
@@ -810,13 +820,15 @@ class Torrent(object):
 
     def finished(self, policy="auto", finished_this_session=True):
         assert self.state == "running", "state not running"
+        self.logger.debug("done downloading, preparing to wrap up")
         # because _finished() calls shutdown(), which invalidates the torrent
         # context, we need to use _rawserver.add_task directly here
-        df = launch_coroutine(_wrap_task(self._rawserver.add_task), self._finished, policy=policy, finished_this_session=finished_this_session)
+        df = launch_coroutine(wrap_task(self._rawserver.add_task), self._finished, policy=policy, finished_this_session=finished_this_session)
         df.addErrback(self.got_exception)
         return df
 
     def _finished(self, policy="auto", finished_this_session=True):
+        self.logger.debug("wrapping up")
         if self.state != "running":
             return
 
@@ -832,8 +844,11 @@ class Torrent(object):
 
         # If we haven't announced yet, normal first announce done later will
         # tell the tracker about seed status.
-        if self._announced:
+        # Only send completed the first time! Torrents transition to finished
+        # everytime.
+        if self._announced and not self.sent_completed:
             self._rerequest_op().announce_finish()
+        self.sent_completed = True
         self._activity = (_("seeding"), 1)
         if self.config['check_hashes']:
             self._save_fastresume()
@@ -891,7 +906,7 @@ class Torrent(object):
             self.logger.debug("successfully paused torrent, moving file")
 
             self.state = "finishing"
-            df = ThreadedDeferred(_wrap_task(self._rawserver.external_add_task),
+            df = ThreadedDeferred(wrap_task(self._rawserver.external_add_task),
                                   move, self.working_path, self.destination_path)
             yield df
             df.getResult()
