@@ -10,10 +10,11 @@
 
 # Written by Bram Cohen and John Hoffman
 
-import sys
 import os
-import signal
 import re
+import sys
+import signal
+import urllib
 import cPickle
 import logging
 import datetime
@@ -41,7 +42,7 @@ from BitTorrent.HTTPHandler import HTTPHandler
 from BTL.parsedir import parsedir
 from BitTorrent.NatCheck import NatCheck
 from BTL.bencode import bencode, bdecode, Bencached
-from urllib import quote, unquote
+from urllib import unquote
 from BTL.exceptions import str_exc
 from BitTorrent import version
 from BitTorrent.prefs import Preferences
@@ -54,6 +55,9 @@ import traceback
 
 NOISY = False
 
+def quote(x):
+    return urllib.quote(x, safe='')
+
 # code duplication because ow.
 MAX_INCOMPLETE = 100
 if os.name == 'nt':
@@ -120,8 +124,8 @@ def isotime():
 http_via_filter = re.compile(' for ([0-9.]+)\Z')
 
 def _get_forwarded_ip(headers):
-    if headers.has_key('http_x_forwarded_for'):
-        header = headers['http_x_forwarded_for']
+    if headers.has_key('x_forwarded_for'):
+        header = headers['x_forwarded_for']
         try:
             x,y = header.split(',')
         except:
@@ -129,16 +133,16 @@ def _get_forwarded_ip(headers):
         if not is_local_ip(x):
             return x
         return y
-    if headers.has_key('http_client_ip'):
-        return headers['http_client_ip']
-    if headers.has_key('http_via'):
-        x = http_via_filter.search(headers['http_via'])
+    if headers.has_key('client_ip'):
+        return headers['client_ip']
+    if headers.has_key('via'):
+        x = http_via_filter.search(headers['via'])
         try:
             return x.group(1)
         except:
             pass
-    if headers.has_key('http_from'):
-        return headers['http_from']
+    if headers.has_key('from'):
+        return headers['from']
     return None
 
 def get_forwarded_ip(headers):
@@ -361,7 +365,7 @@ class Tracker(object):
                     nf = nf + 1
                     if self.allowed is not None and self.show_names:
                         if self.allowed.has_key(infohash):
-                            sz = self.allowed[infohash]['length']  # size
+                            sz = self.allowed[infohash][1].total_bytes # size
                             ts = ts + sz
                             szt = sz * n   # Transferred for this torrent
                             tt = tt + szt
@@ -735,7 +739,7 @@ class Tracker(object):
 
     def natchecklog(self, peerid, ip, port, result):
         print isotime(), '"!natcheck-%s:%i" %s %i 0 - -' % (
-            ip, quote(peerid), port, result)
+            ip, port, quote(peerid), result)
 
     def connectback_result(self, result, downloadid, peerid, ip, port):
         record = self.downloads.get(downloadid, {}).get(peerid)
