@@ -12,10 +12,10 @@
 
 # Written by Bram Cohen
 
+app_name = "BitTorrent"
 if __name__ == '__main__':
-    from BitTorrent.platform import install_translation
-    install_translation()
-
+    from BitTorrent.translation import _
+    
 import os
 from sys import argv
 from BitTorrent.track import track
