@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -18,6 +18,7 @@ class IDChecker:
         self.id = id
 
 class KNodeBase(Node):
+    __slots__= ('cfa', 'table')
     def __init__(self, cfa):
         Node.__init__(self)
         self.cfa = cfa
@@ -41,7 +42,7 @@ class KNodeBase(Node):
 
     def errBack(self, err):
         self.table.table.nodeFailed(self)
-        return err
+        return err[0]
         
     def ping(self, id):
         df = self.conn().sendRequest('ping', {"id":id})
