@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -9,11 +9,12 @@
 # License.
 
 import khash
-from BitTorrent.platform import bttime as time
+from BTL.platform import bttime as time
 from types import *
 
-class Node:
+class Node(object):
     """encapsulate contact info"""
+    __slots__ = ('fails','lastSeen','invalid','id','host','port','age')
     def __init__(self):
         self.fails = 0
         self.lastSeen = 0
@@ -49,6 +50,9 @@ class Node:
     
     def senderDict(self):
         return self._senderDict
+
+    def __hash__(self):
+        return self.id.__hash__()
     
     def __repr__(self):
         return ">node <%s> %s<" % (self.id.encode('base64')[:4], (self.host, self.port))
