@@ -1,5 +1,5 @@
 # The contents of this file are subject to the BitTorrent Open Source License
-# Version 1.0 (the License).  You may not copy or use this file, in either
+# Version 1.1 (the License).  You may not copy or use this file, in either
 # source code or executable form, except in compliance with the License.  You
 # may obtain a copy of the License at http://www.bittorrent.com/license/.
 #
@@ -9,7 +9,7 @@
 # License.
 
 import unittest
-from BitTorrent.platform import bttime
+from BTLR.platform import bttime
 from time import sleep
 
 from kstore import KStore
@@ -73,3 +73,19 @@ class BasicTests(unittest.TestCase):
         self.k['foo'] = 'bar'
         self.k['foo'] = 'bar'
         self.assertEqual(self.k['foo'], ['bar'])
+
+    def testSample(self):
+        for i in xrange(2):
+            self.k['foo'] = i
+        l = self.k.sample('foo', 5)
+        l.sort()
+        self.assertEqual(l, [0, 1])
+
+        for i in xrange(10):
+            for i in xrange(10):
+                self.k['bar'] = i
+            l = self.k.sample('bar', 5)
+            self.assertEqual(len(l), 5)
+            for i in xrange(len(l)):
+                self.assert_(l[i] not in l[i+1:])
+        
